/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.project.api.env;

import cn.gjing.tools.auth.annotation.RequiredPermissions;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.project.api.env.service.CpProjectEnvDsApiService;
import net.northking.cctp.project.dto.env.CpProjectEnvDsDetailDto;
import net.northking.cctp.project.dto.env.CpProjectEnvDsQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "系统环境数据源信息")
@RequestMapping("/v1")
@RestController
public class CpProjectEnvDsQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(CpProjectEnvDsQueryCtrl.class);

  @Autowired
  private CpProjectEnvDsApiService apiService;

  /**
   * 查询用户列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表")
  @RequiredPermissions({"ui-queryEnvDss","ui-configureEnvDss"})
  @PostMapping(value = "/paging/query/CpProjectEnvDss",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<CpProjectEnvDsDetailDto>> pagingQuery(
      @RequestBody QueryByPage<CpProjectEnvDsQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<CpProjectEnvDsDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<CpProjectEnvDsDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

}


