/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.db.entity.BugStatusStatistics;
import net.northking.cctp.dw.dto.*;

import java.util.List;

/**
 * 缺陷状态统计Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugStatusStatisticsApiService extends ExcelService
{
  /**
   * 查询缺陷状态统计列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugStatusStatisticsDetailDto> pagingQuery(QueryByPage<BugStatusStatisticsQueryDto> queryByPage);

  /**
   * 新增缺陷状态统计
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugStatusStatisticsAddDto dto);

  /**
   * 更新缺陷状态统计
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugStatusStatisticsUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取缺陷状态统计
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugStatusStatisticsDetailDto findByPK(String uuid);

  /**
   * 删除缺陷状态统计
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugStatusStatisticsAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugStatusStatisticsUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<BugStatusStatisticsDetailDto> listQuery(ChartDataGeneQueryDto chartDataGeneQueryDto);

    List<BugStatusStatistics> queryUnresolvedBugModuleData(String projectId,String tenantId);

  List<BugStatusStatistics> queryTop5TypeBugDataByTenantId(String tenantId,List<String> projectIds);
}
