/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.saas.dto.tenant;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import net.northking.cctp.saas.db.entity.SaasTenantAdmin;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
* Restful接口--资源详情--数据传输对象：租户
*
* createdate:  2021-03-02 16:33:19  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "租户")
public class SaaSTenantDetailDto implements Serializable
{
  /**
   * 主键<br>
   */
  @ApiModelProperty("主键")
  @Size(max = 32, message = "主键-id: 数据长度不能 > 32")
  private String id;
  /**
   * 租户代码<br>
   */
  @ApiModelProperty("租户代码")
  private String tenantCode;
  /**
   * 租户名称<br>
   */
  @ApiModelProperty("租户名称")
  private String tenantName;
  /**
   * 描述说明<br>
   */
  @ApiModelProperty("描述说明")
  @Size(max = 512, message = "描述说明-memo: 数据长度不能 > 512")
  private String memo;
  /**
   * 数据隔离级别<br>
   */
  @ApiModelProperty("数据隔离级别")
  private String isolationLevel;
  /**
   * 创建人<br>
   */
  @ApiModelProperty("创建人")
  private String createdBy;
  /**
   * 创建时间<br>
   */
  @ApiModelProperty("创建时间")
  private Date createdTime;
  /**
   *  租户状态<br>
   */
  @ApiModelProperty("租户状态")
  private String tenantStatus;
  /**
   *  租户联系电话<br>
   */
  @ApiModelProperty("租户联系电话")
  private String tenantPhone;
  /**
   *  租户邮箱<br>
   */
  @ApiModelProperty("租户邮箱")
  private String tenantEmail;
  /**
   *  租户类型<br>
   */
  @ApiModelProperty("租户类型")
  private String tenantType;
  /**
   *  license数量<br>
   */
  @ApiModelProperty("license数量")
  private Integer licenseCount;
  /**
   *  启动/禁用<br>
   */
  @ApiModelProperty("启动/禁用")
  private Boolean enable;
  /**
   *  租户管理员
   */
  @ApiModelProperty("租户管理员")
  private List<SaasTenantAdmin> adminList;
  /**
   * 是否删除
   */
  @ApiModelProperty("是否删除")
  public Boolean isDeleted;
  /**
   * 最大在线用户数量<br>
   */
  @ApiModelProperty("最大在线用户数量")
  private Integer maxOnlineUser;

  @ApiModelProperty("租户结束时间")
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  private Date endTime;

  @ApiModelProperty("二级菜单位置")
  private Boolean position;

  @ApiModelProperty("是否展示系统空间")
  private Boolean projectSpace;

  @ApiModelProperty("租户管理员账号")
  private String tenantCreated;

  @ApiModelProperty("租户logo的id")
  private String logoId;

  public String getLogoId() {
    return logoId;
  }

  public void setLogoId(String logoId) {
    this.logoId = logoId;
  }


  public String getTenantCreated() {
    return tenantCreated;
  }

  public void setTenantCreated(String tenantCreated) {
    this.tenantCreated = tenantCreated;
  }

  public Boolean getPosition() {
    return position;
  }

  public void setPosition(Boolean position) {
    this.position = position;
  }

  public Boolean getProjectSpace() {
    return projectSpace;
  }

  public void setProjectSpace(Boolean projectSpace) {
    this.projectSpace = projectSpace;
  }

  public Date getEndTime() {
    return endTime;
  }

  public void setEndTime(Date endTime) {
    this.endTime = endTime;
  }

  public String getMemo() {
    return memo;
  }

  public void setMemo(String memo) {
    this.memo = memo;
  }

  public Boolean getDeleted() {
    return isDeleted;
  }

  public void setDeleted(Boolean deleted) {
    isDeleted = deleted;
  }

  public List<SaasTenantAdmin> getAdminList() {
    return adminList;
  }

  public void setAdminList(List<SaasTenantAdmin> adminList) {
    this.adminList = adminList;
  }

  public String getId() {
    return id;
  }

  public void setId(String id) {
    this.id = id;
  }

  public String getTenantCode() {
    return tenantCode;
  }

  public void setTenantCode(String tenantCode) {
    this.tenantCode = tenantCode;
  }

  public String getTenantName() {
    return tenantName;
  }

  public void setTenantName(String tenantName) {
    this.tenantName = tenantName;
  }

  public String getIsolationLevel() {
    return isolationLevel;
  }

  public void setIsolationLevel(String isolationLevel) {
    this.isolationLevel = isolationLevel;
  }

  public String getCreatedBy() {
    return createdBy;
  }

  public void setCreatedBy(String createdBy) {
    this.createdBy = createdBy;
  }

  public Date getCreatedTime() {
    return createdTime;
  }

  public void setCreatedTime(Date createdTime) {
    this.createdTime = createdTime;
  }

  public String getTenantStatus() {
    return tenantStatus;
  }

  public void setTenantStatus(String tenantStatus) {
    this.tenantStatus = tenantStatus;
  }

  public String getTenantPhone() {
    return tenantPhone;
  }

  public void setTenantPhone(String tenantPhone) {
    this.tenantPhone = tenantPhone;
  }

  public String getTenantEmail() {
    return tenantEmail;
  }

  public void setTenantEmail(String tenantEmail) {
    this.tenantEmail = tenantEmail;
  }

  public String getTenantType() {
    return tenantType;
  }

  public void setTenantType(String tenantType) {
    this.tenantType = tenantType;
  }

  public Integer getLicenseCount() {
    return licenseCount;
  }

  public void setLicenseCount(Integer licenseCount) {
    this.licenseCount = licenseCount;
  }

  public Boolean getEnable() {
    return enable;
  }

  public void setEnable(Boolean enable) {
    this.enable = enable;
  }

  public Integer getMaxOnlineUser() {
    return maxOnlineUser;
  }

  public void setMaxOnlineUser(Integer maxOnlineUser) {
    this.maxOnlineUser = maxOnlineUser;
  }
}